import { join } from 'path';
import { validateFunction, getProjectRoot, getConfig } from '@midwayjs/hooks-core';
import { createHttpRequest, close, createFunctionApp as createFunctionApp$1, createApp as createApp$1 } from '@midwayjs/mock';

async function createAppImplementation(options, isFunction) {
  var _options$root;

  const root = getProjectRoot((_options$root = options === null || options === void 0 ? void 0 : options.root) !== null && _options$root !== void 0 ? _options$root : global.testPath);
  const config = getConfig(root);

  if (options !== null && options !== void 0 && options.root) {
    options === null || options === void 0 ? true : delete options.root;
  }

  const cwd = process.cwd();
  process.chdir(root);
  const creator = isFunction ? createFunctionApp$1 : createApp$1;
  const app = await creator(root, Object.assign({
    baseDir: join(root, config.source)
  }, options));
  process.chdir(cwd);
  return new HooksApplication(app, config);
}

async function createApp(options) {
  return createAppImplementation(options, false);
}
async function createFunctionApp(options) {
  return createAppImplementation(options, true);
}
class HooksApplication {
  constructor(app, config) {
    this.app = void 0;
    this.config = void 0;
    this.app = app;
    this.config = config;
  }

  async runFunction(fn, ...args) {
    validateFunction(fn, 'fn');
    const response = await this.request(fn, ...args);

    if (!response.ok) {
      throw response.body;
    }

    if (response.type === 'application/json') {
      return response.body;
    }

    return response.text;
  }

  request(fn, ...args) {
    validateFunction(fn, 'fn');
    const supertest = createHttpRequest(this.app);

    if (args.length === 0) {
      return supertest.get(fn._param.url);
    }

    return supertest.post(fn._param.url).send({
      args
    }).set('Content-Type', 'application/json');
  }

  async close() {
    await close(this.app);
  }

}

export { HooksApplication, createApp, createFunctionApp };
